precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputTexture;
uniform sampler2D inputTexture2;
uniform vec3 hsl;
float rgb2gray(vec3 color){
    return dot(color, vec3(0.299, 0.587, 0.114));
}
float rgb_max(vec3 rgb) {
    return max(rgb.r, max(rgb.g, rgb.b));
}
float rgb_min(vec3 rgb) {
    return min(rgb.r, min(rgb.g, rgb.b));
}
vec3 rgb2hsl(vec3 rgb) {
    float rgbmax = rgb_max(rgb);
    float rgbmin = rgb_min(rgb);
    vec3 hsl = vec3(0.);
    hsl.z = (rgbmax + rgbmin) / 2.;
    if (rgbmax == rgbmin) {
        hsl.y = 0.;
        hsl.x = -1.;
    } else {
        float delta = rgbmax-rgbmin;
        hsl.y = mix(delta/(2.-rgbmax-rgbmin), delta/(rgbmax+rgbmin), step(hsl.z, .5));
        if (delta == 0.) delta = 1.;
        if (rgb.r == rgbmax) hsl.x = (rgb.g-rgb.b)/delta;
        else if (rgb.g == rgbmax) hsl.x = 2. + (rgb.b-rgb.r)/delta;
        else hsl.x = 4.+(rgb.r-rgb.g)/delta;
        hsl.x /= 6.;
        hsl.x = mix(hsl.x+1., hsl.x, step(0., hsl.x));
    }
    return hsl;
}
float hsl_value(float n1, float n2, float hue) {
    if (hue < 0.){
        hue = hue + 6.;
    } else if (hue > 6.){
        hue = hue - 6.;
    }
    if (hue > 4.){
        return n1;
    } else if (hue > 3.){
        return n1 + (n2 - n1)*(4.-hue);
    } else if (hue > 1.){
        return n2;
    } else {
        return n1 + (n2 - n1) * hue;
    }
}
vec3 hsl2rgb(vec3 hsl) {
    vec3 rgb = vec3(0.);
    if (hsl.y == 0.) {
        rgb.r = hsl.z;
        rgb.g = hsl.z;
        rgb.b = hsl.z;
    } else {
        float m2 = mix(hsl.z + hsl.y - hsl.z*hsl.y, hsl.z*(1.+hsl.y), step(hsl.z, .5));
        float m1 = 2.*hsl.z-m2;
        rgb.r = hsl_value(m1, m2, hsl.x * 6. + 2.);
        rgb.g = hsl_value(m1, m2, hsl.x * 6.);
        rgb.b = hsl_value(m1, m2, hsl.x * 6. - 2.);
    }
    return rgb;
}
void main() {
    vec4 color = texture2D(inputTexture, textureCoordinate);
    vec4 mask = texture2D(inputTexture2, textureCoordinate);
    vec3 srcHsl = rgb2hsl(color.rgb);
    vec3 dstHsl = vec3(hsl.x, srcHsl.y * 2., srcHsl.z + 2. * (hsl.z - 0.5) * (srcHsl.z * 0.5 + 0.5));
    color.rgb = hsl2rgb(dstHsl);
    color.a = color.a * rgb2gray(mask.rgb);
    gl_FragColor = color;
}