precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputTexture;
uniform sampler2D inputTexture2;
uniform float skinStrength;

float doMix(float src, float dst) {
    if (dst <= 0.5) {
        return src - (1.0 - 2.0 * dst) * src * (1.0 - src);
    } else {
        float beta= 0.0;
        if (src <= 0.25) {
            beta = ((16.0 * src - 12.0) * src + 4.0) * src;
        } else {
            beta = sqrt(src);
        }
        return src + (2.0 * dst - 1.0) * (beta - src);
    }
}

vec3 blend(vec4 base, vec4 overlay) {
    vec3 baseRGB = base.rgb;
    vec3 overlayRGB = overlay.rgb;
    float r = doMix(baseRGB.r, overlayRGB.r);
    float g = doMix(baseRGB.g, overlayRGB.g);
    float b = doMix(baseRGB.b, overlayRGB.b);
    vec3 color = clamp(vec3(r, g, b), 0.0, 1.0);
    return (1.0 - overlay.a) * base.rgb + (1.0 - base.a) * overlay.rgb + base.a * overlay.a * color;
}

void main() {
    vec4 texture1Color = texture2D(inputTexture, textureCoordinate);
    vec4 texture2Color = texture2D(inputTexture2, textureCoordinate);
    texture2Color.a = texture2Color.a * skinStrength;
    float alpha = texture1Color.a + texture2Color.a - texture1Color.a * texture2Color.a;
    gl_FragColor = vec4(blend(texture1Color, texture2Color), alpha);
}